@@ -1,7 +1,7 @@
-/* $Id: lex.c,v 1.26 2005/10/16 15:26:51 dijkstra Exp $ */
+/* $Id: lex.c,v 1.28 2007/02/11 20:07:31 dijkstra Exp $ */
 
 /*
- * Copyright (c) 2001-2005 Willem Dijkstra
+ * Copyright (c) 2001-2007 Willem Dijkstra
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -74,6 +74,7 @@ static struct {
     { "debug", LXT_DEBUG },
     { "df", LXT_DF },
     { "every", LXT_EVERY },
+    { "from", LXT_FROM },
     { "if", LXT_IF },
     { "in", LXT_IN },
     { "io", LXT_IO },
@@ -105,20 +106,20 @@ parse_token(const char *cp)
     u_int i;
 
     for (i = 0; keywords[i].name; i++)
-	if (strcasecmp(cp, keywords[i].name) == 0)
-	    return keywords[i].opcode;
+        if (strcasecmp(cp, keywords[i].name) == 0)
+            return keywords[i].opcode;
 
     return LXT_BADTOKEN;
 }
 /* Return the ascii representation of an opcode */
-const char *
+char *
 parse_opcode(const int op)
 {
     u_int i;
 
     for (i = 0; keywords[i].name; i++)
-	if (keywords[i].opcode == op)
-	    return keywords[i].name;
+        if (keywords[i].opcode == op)
+            return (char *) keywords[i].name;
 
     return NULL;
 }
@@ -129,17 +130,17 @@ lex_readline(struct lex *l)
     char *bp;
 
     if (l == NULL)
-	return 0;
+        return 0;
 
     bp = l->buffer;
 
     if (l->buffer) {
-	l->curpos = 0;
-	l->endpos = 0;
+        l->curpos = 0;
+        l->endpos = 0;
     } else {
-	l->bsize = _POSIX2_LINE_MAX;
-	l->buffer = xmalloc(l->bsize);
-	bp = l->buffer;
+        l->bsize = _POSIX2_LINE_MAX;
+        l->buffer = xmalloc(l->bsize);
+        bp = l->buffer;
     }
 
     l->endpos = read(l->fh, bp, (l->buffer + l->bsize) - bp);
@@ -151,14 +152,14 @@ void
 lex_copychar(struct lex *l)
 {
     if (l == NULL)
-	return;
+        return;
 
     l->token[l->tokpos] = l->buffer[l->curpos];
 
     if (++l->tokpos >= _POSIX2_LINE_MAX) {
-	l->token[_POSIX2_LINE_MAX - 1] = '\0';
-	fatal("%.200s:%d: parse error at '%.200s'", l->filename, l->cline, l->token);
-	/* NOT REACHED */
+        l->token[_POSIX2_LINE_MAX - 1] = '\0';
+        fatal("%.200s:%d: parse error at '%.200s'", l->filename, l->cline, l->token);
+        /* NOT REACHED */
     }
 }
 /* Get next char, read next line if needed */
@@ -166,16 +167,16 @@ int
 lex_nextchar(struct lex *l)
 {
     if (l == NULL)
-	return 0;
+        return 0;
 
     l->curpos++;
 
     if (l->curpos >= l->endpos)
-	if (!lex_readline(l))
-	    return 0;
+        if (!lex_readline(l))
+            return 0;
 
     if (l->buffer[l->curpos] == '\n')
-	l->cline++;
+        l->cline++;
 
     return 1;
 }
@@ -184,7 +185,7 @@ void
 lex_termtoken(struct lex *l)
 {
     if (l == NULL)
-	return;
+        return;
 
     l->token[l->tokpos] = l->token[_POSIX2_LINE_MAX - 1] = '\0';
     l->tokpos = 0;
@@ -194,7 +195,7 @@ void
 lex_ungettoken(struct lex *l)
 {
     if (l == NULL)
-	return;
+        return;
 
     l->unget = 1;
 }
@@ -203,12 +204,12 @@ int
 lex_nexttoken(struct lex *l)
 {
     if (l == NULL)
-	return 0;
+        return 0;
 
     /* return same token as last time if it has been pushed back */
     if (l->unget) {
-	l->unget = 0;
-	return 1;
+        l->unget = 0;
+        return 1;
     }
 
     l->op = LXT_BADTOKEN;
@@ -217,88 +218,88 @@ lex_nexttoken(struct lex *l)
 
     /* find first non whitespace */
     while (l->buffer[l->curpos] == ' ' ||
-	   l->buffer[l->curpos] == '\t' ||
-	   l->buffer[l->curpos] == '\r' ||
-	   l->buffer[l->curpos] == '\n' ||
-	   l->buffer[l->curpos] == '\0' ||
-	   l->buffer[l->curpos] == '#') {
-	/* flush rest of line if comment */
-	if (l->buffer[l->curpos] == '#') {
-	    while (l->buffer[l->curpos] != '\n')
-		if (!lex_nextchar(l))
-		    return 0;
-	} else if (!lex_nextchar(l))
-	    return 0;
+           l->buffer[l->curpos] == '\t' ||
+           l->buffer[l->curpos] == '\r' ||
+           l->buffer[l->curpos] == '\n' ||
+           l->buffer[l->curpos] == '\0' ||
+           l->buffer[l->curpos] == '#') {
+        /* flush rest of line if comment */
+        if (l->buffer[l->curpos] == '#') {
+            while (l->buffer[l->curpos] != '\n')
+                if (!lex_nextchar(l))
+                    return 0;
+        } else if (!lex_nextchar(l))
+            return 0;
     }
 
     l->type = LXY_STRING;
 
     /* "delimited string" */
     if (l->buffer[l->curpos] == '"') {
-	if (!lex_nextchar(l)) {
-	    warning("%.200s:%d: unbalanced '\"'", l->filename, l->cline);
-	    return 0;
-	}
-	while (l->buffer[l->curpos] != '"') {
-	    lex_copychar(l);
-	    if (!lex_nextchar(l)) {
-		warning("%.200s:%d: unbalanced '\"'", l->filename, l->cline);
-		return 0;
-	    }
-	}
-	lex_termtoken(l);
-	lex_nextchar(l);
-	return 1;
+        if (!lex_nextchar(l)) {
+            warning("%.200s:%d: unbalanced '\"'", l->filename, l->cline);
+            return 0;
+        }
+        while (l->buffer[l->curpos] != '"') {
+            lex_copychar(l);
+            if (!lex_nextchar(l)) {
+                warning("%.200s:%d: unbalanced '\"'", l->filename, l->cline);
+                return 0;
+            }
+        }
+        lex_termtoken(l);
+        lex_nextchar(l);
+        return 1;
     }
 
     /* 'delimited string' */
     if (l->buffer[l->curpos] == '\'') {
-	if (!lex_nextchar(l)) {
-	    warning("%.200s:%d: unbalanced \"\'\"", l->filename, l->cline);
-	    return 0;
-	}
-	while (l->buffer[l->curpos] != '\'') {
-	    lex_copychar(l);
-	    if (!lex_nextchar(l)) {
-		warning("%.200s:%d: unbalanced \"\'\"", l->filename, l->cline);
-		return 0;
-	    }
-	}
-	lex_termtoken(l);
-	lex_nextchar(l);
-	return 1;
+        if (!lex_nextchar(l)) {
+            warning("%.200s:%d: unbalanced \"\'\"", l->filename, l->cline);
+            return 0;
+        }
+        while (l->buffer[l->curpos] != '\'') {
+            lex_copychar(l);
+            if (!lex_nextchar(l)) {
+                warning("%.200s:%d: unbalanced \"\'\"", l->filename, l->cline);
+                return 0;
+            }
+        }
+        lex_termtoken(l);
+        lex_nextchar(l);
+        return 1;
     }
 
     /* one char keyword */
     if (strchr(KW_OPS, l->buffer[l->curpos])) {
-	lex_copychar(l);
-	lex_termtoken(l);
-	l->op = parse_token(l->token);
-	lex_nextchar(l);
-	return 1;
+        lex_copychar(l);
+        lex_termtoken(l);
+        l->op = parse_token(l->token);
+        lex_nextchar(l);
+        return 1;
     }
 
     /* single keyword */
     while (l->buffer[l->curpos] != ' ' &&
-	   l->buffer[l->curpos] != '\t' &&
-	   l->buffer[l->curpos] != '\r' &&
-	   l->buffer[l->curpos] != '\n' &&
-	   l->buffer[l->curpos] != '\0' &&
-	   l->buffer[l->curpos] != '#' &&
-	   (strchr(KW_OPS, l->buffer[l->curpos]) == NULL)) {
-	lex_copychar(l);
-	if (!lex_nextchar(l))
-	    break;
+           l->buffer[l->curpos] != '\t' &&
+           l->buffer[l->curpos] != '\r' &&
+           l->buffer[l->curpos] != '\n' &&
+           l->buffer[l->curpos] != '\0' &&
+           l->buffer[l->curpos] != '#' &&
+           (strchr(KW_OPS, l->buffer[l->curpos]) == NULL)) {
+        lex_copychar(l);
+        if (!lex_nextchar(l))
+            break;
     }
     lex_termtoken(l);
     l->op = parse_token(l->token);
 
     /* number */
     if (l->token[0] >= '0' && l->token[0] <= '9') {
-	if (strlen(l->token) == strspn(l->token, "0123456789")) {
-	    l->type = LXY_NUMBER;
-	    l->value = strtol(l->token, NULL, 10);
-	}
+        if (strlen(l->token) == strspn(l->token, "0123456789")) {
+            l->type = LXY_NUMBER;
+            l->value = strtol(l->token, NULL, 10);
+        }
     }
     return 1;
 }
@@ -315,10 +316,10 @@ open_lex(const char *filename)
     l->token = xmalloc(_POSIX2_LINE_MAX);
 
     if ((l->fh = open(l->filename, O_RDONLY)) < 0) {
-	warning("could not open file \"%.200s\":%.200s",
-		l->filename, strerror(errno));
-	close_lex(l);
-	return NULL;
+        warning("could not open file \"%.200s\":%.200s",
+                l->filename, strerror(errno));
+        close_lex(l);
+        return NULL;
     }
 
     lex_readline(l);
@@ -331,13 +332,13 @@ rewind_lex(struct lex *l)
     off_t filepos;
 
     if (l == NULL)
-	return;
+        return;
 
     reset_lex(l);
 
     if ((filepos = lseek(l->fh, (off_t)0, SEEK_SET)) == -1) {
-	warning("could not rewind file '%.200s':%.200s",
-		l->filename, strerror(errno));
+        warning("could not rewind file '%.200s':%.200s",
+                l->filename, strerror(errno));
     }
 }
 /* Reset lexer to start of file defaults */
@@ -345,7 +346,7 @@ void
 reset_lex(struct lex *l)
 {
     if (l == NULL)
-	return;
+        return;
 
     l->cline = 1;
     l->curpos = 0;
@@ -361,14 +362,14 @@ void
 close_lex(struct lex *l)
 {
     if (l == NULL)
-	return;
+        return;
 
     if (l->fh)
-	close(l->fh);
+        close(l->fh);
     if (l->buffer)
-	xfree(l->buffer);
+        xfree(l->buffer);
     if (l->token)
-	xfree(l->token);
+        xfree(l->token);
     xfree(l);
 }
 /* Signal a parse error */
@@ -376,5 +377,5 @@ void
 parse_error(struct lex *l, const char *s)
 {
     warning("%.200s:%d: expected '%.200s' found '%.8s'",
-	    l->filename, l->cline, s, l->token);
+            l->filename, l->cline, s, l->token);
 }
