@@ -1,4 +1,4 @@
-/* $Id: platform.h,v 1.4 2007/02/11 20:07:32 dijkstra Exp $ */
+/* $Id: platform.h,v 1.7 2007/07/09 13:19:46 dijkstra Exp $ */
 
 #ifndef _CONF_LINUX_H
 #define _CONF_LINUX_H
@@ -19,18 +19,29 @@ union semun {
         int val;
 };
 
-#define CPUSTATES 4
-#define CP_USER 0
-#define CP_NICE 1
-#define CP_SYS  2
-#define CP_IDLE 3
+#ifdef LONG_LONG_MAX
+#define QUAD_MAX LONG_LONG_MAX
+#endif
+#ifndef QUAD_MAX
+#define QUAD_MAX     (0x7fffffffffffffffLL)
+#endif
+
+#define CPUSTATES    8
+#define CP_USER      0
+#define CP_NICE      1
+#define CP_SYS       2
+#define CP_IDLE      3
+#define CP_IOWAIT    4
+#define CP_HARDIRQ   5
+#define CP_SOFTIRQ   6
+#define CP_STEAL     7
 
 union stream_parg {
     struct {
-        long time[CPUSTATES];
-        long old[CPUSTATES];
-        long diff[CPUSTATES];
-        int states[CPUSTATES];
+        int64_t time[CPUSTATES];
+        int64_t old[CPUSTATES];
+        int64_t diff[CPUSTATES];
+        int64_t states[CPUSTATES];
         char name[6];
     } cp;
 };
