@@ -1,11 +1,11 @@
-/* $Id: sm_cpu.c,v 1.5 2007/02/11 20:07:32 dijkstra Exp $ */
+/* $Id: sm_cpu.c,v 1.7 2007/07/09 12:54:18 dijkstra Exp $ */
 
 /* The author of this code is Willem Dijkstra (wpd@xs4all.nl).
  *
  * The percentages function was written by William LeFebvre and is part
  * of the 'top' utility. His copyright statement is below.
  *
- * Copyright (c) 2001-2005 Willem Dijkstra
+ * Copyright (c) 2001-2007 Willem Dijkstra
  * All rights reserved.
  *
  * Redistribution and use in source and binary forms, with or without
@@ -69,7 +69,7 @@
 #include "xmalloc.h"
 
 __BEGIN_DECLS
-int percentages(int, int *, long *, long *, long *);
+int percentages(int, int64_t *, int64_t *, int64_t *, int64_t *);
 __END_DECLS
 
 /* Globals for this module all start with cp_ */
@@ -85,13 +85,10 @@ static int cp_maxsize = 0;
  *      useful on BSD mchines for calculating cpu state percentages.
  */
 int
-percentages(int cnt, int *out, register long *new, register long *old, long *diffs)
+percentages(int cnt, int64_t *out, int64_t *new, int64_t *old, int64_t *diffs)
 {
-    register int i;
-    register long change;
-    register long total_change;
-    register long *dp;
-    long half_total;
+    int64_t change, total_change, *dp, half_total;
+    int i;
 
     /* initialization */
     total_change = 0;
@@ -101,7 +98,7 @@ percentages(int cnt, int *out, register long *new, register long *old, long *dif
     for (i = 0; i < cnt; i++) {
         if ((change = *new - *old) < 0) {
             /* this only happens when the counter wraps */
-            change = ((unsigned int) *new - (unsigned int) *old);
+            change = (QUAD_MAX - *old) + *new;
         }
         total_change += (*dp++ = change);
         *old++ = *new++;
@@ -117,7 +114,7 @@ percentages(int cnt, int *out, register long *new, register long *old, long *dif
         *out++ = ((*diffs++ * 1000 + half_total) / total_change);
 
     /* return the total in case the caller wants to use it */
-    return total_change;
+    return (total_change);
 }
 
 void
@@ -188,11 +185,15 @@ get_cpu(char *symon_buf, int maxlen, struct stream *st)
     }
 
     line += strlen(st->parg.cp.name);
-    if (4 > sscanf(line, "%lu %lu %lu %lu\n",
-                   &st->parg.cp.time[CP_USER],
-                   &st->parg.cp.time[CP_NICE],
-                   &st->parg.cp.time[CP_SYS],
-                   &st->parg.cp.time[CP_IDLE])) {
+    if (CPUSTATES > sscanf(line, "%llu %llu %llu %llu %llu %llu %llu %llu\n",
+                           &st->parg.cp.time[CP_USER],
+                           &st->parg.cp.time[CP_NICE],
+                           &st->parg.cp.time[CP_SYS],
+                           &st->parg.cp.time[CP_IDLE],
+                           &st->parg.cp.time[CP_IOWAIT],
+                           &st->parg.cp.time[CP_HARDIRQ],
+                           &st->parg.cp.time[CP_SOFTIRQ],
+                           &st->parg.cp.time[CP_STEAL])) {
         warning("could not parse cpu statistics for %.200s", &st->parg.cp.name);
         return 0;
     }
@@ -204,6 +205,9 @@ get_cpu(char *symon_buf, int maxlen, struct stream *st)
                   (double) (st->parg.cp.states[CP_USER] / 10.0),
                   (double) (st->parg.cp.states[CP_NICE] / 10.0),
                   (double) (st->parg.cp.states[CP_SYS] / 10.0),
-                  (double) (0),
+                  (double) (st->parg.cp.states[CP_IOWAIT] +
+                            st->parg.cp.states[CP_HARDIRQ] +
+                            st->parg.cp.states[CP_SOFTIRQ] +
+                            st->parg.cp.states[CP_STEAL]) / 10.0,
                   (double) (st->parg.cp.states[CP_IDLE] / 10.0));
 }
