@@ -1,4 +1,4 @@
-/* $Id: readconf.c,v 1.25 2007/02/11 20:07:32 dijkstra Exp $ */
+/* $Id: readconf.c,v 1.27 2007/07/09 11:43:13 dijkstra Exp $ */
 
 /*
  * Copyright (c) 2001-2005 Willem Dijkstra
@@ -30,6 +30,8 @@
  *
  */
 
+#include <sys/types.h>
+
 #include <string.h>
 
 #include "conf.h"
@@ -201,12 +203,13 @@ read_monitor(struct muxlist * mul, struct lex * l)
 
     if (l->op == LXT_FROM) {
         lex_nexttoken(l);
+        /* check to see host is resolvable, result is discarded */
         if (!getip(l->token, AF_INET) && !getip(l->token, AF_INET6)) {
             warning("%.200s:%d: could not resolve '%.200s'",
                 l->filename, l->cline, l->token);
             return 0;
         }
-        mux->localaddr = xstrdup((const char *) &res_host);
+        mux->localaddr = xstrdup(l->token);
         lex_nexttoken(l);
     }
 
